@@ -77,6 +77,8 @@ static SFLogin *SFlogins;
 
 static int SFtwiddle = 0;
 
+void SFsetText(char *path);
+
 int
 SFchdir(path)
 	char	*path;
@@ -713,6 +715,10 @@ SFbuttonPressList(w, n, event)
 	int			n;
 	XButtonPressedEvent	*event;
 {
+	int dir = 0;
+	if(event->button == Button4) dir = -2; // kludge to indicate relative motion
+	if(event->button == Button5) dir = -1;
+	if(dir) SFvSliderMovedCallback(w, n, dir); else
 	SFbuttonPressed = 1;
 }
 
@@ -724,7 +730,9 @@ SFbuttonReleaseList(w, n, event)
 	XButtonReleasedEvent	*event;
 {
 	SFDir	*dir;
+	static int lastClick;
 
+	if(event->button == Button4 || event->button == Button5) return; // [HGM] mouse wheel does not select
 	SFbuttonPressed = 0;
 
 	if (SFcurrentInvert[n] != -1) {
@@ -738,7 +746,9 @@ SFbuttonReleaseList(w, n, event)
 			dir->entries[dir->vOrigin + SFcurrentInvert[n]].shown
 		);
 		SFmotionList(w, n, (XMotionEvent *) event);
+		if(lastClick == 256*n + SFcurrentInvert[n]) SFstatus = SEL_FILE_OK; // [HGM] double click implies OK
 	}
+	lastClick = 256*n + SFcurrentInvert[n];
 }
 
 static int
