@@ -5,7 +5,7 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -60,6 +60,7 @@ typedef VOIDSTAR ProcRef;
 #define NoProc ((ProcRef) 0)
 typedef VOIDSTAR InputSourceRef;
 
+char *T_ P((char *s));
 void ModeHighlight P((void));
 void SetICSMode P((void));
 void SetGNUMode P((void));
@@ -90,6 +91,10 @@ void CommentPopDown P((void));
 void EditCommentPopUp P((int index, String title, String text));
 void ErrorPopDown P((void));
 int  EventToSquare P((int x, int limit));
+void DrawSeekAxis P(( int x, int y, int xTo, int yTo ));
+void DrawSeekBackground P(( int left, int top, int right, int bottom ));
+void DrawSeekText P((char *buf, int x, int y));
+void DrawSeekDot P((int x, int y, int color));
 
 void RingBell P((void));
 void PlayIcsWinSound P((void));
@@ -127,6 +132,7 @@ void PromotionPopUp P((void));
 void DragPieceBegin P((int x, int y));
 void DragPieceEnd P((int x, int y));
 void LeftClick P((ClickType c, int x, int y));
+int  RightClick P((ClickType c, int x, int y, int *col, int *row));
 
 int StartChildProcess P((char *cmdLine, char *dir, ProcRef *pr));
 void DestroyChildProcess P((ProcRef pr, int/*boolean*/ signal));
@@ -154,6 +160,15 @@ void CmailSigHandlerCallBack P((InputSourceRef isr, VOIDSTAR closure,
 				char *buf, int count, int error));
 
 extern ProcRef cmailPR;
+extern int shiftKey;
+
+/* in xgamelist.c or winboard.c */
+void GLT_ClearList();
+void GLT_DeSelectList();
+void GLT_AddToList( char *name );
+Boolean GLT_GetFromList( int index, char *name );
+
+extern char lpUserGLT[];
 
 /* these are in wgamelist.c */
 void GameListPopUp P((FILE *fp, char *filename));
@@ -162,7 +177,7 @@ void GameListHighlight P((int index));
 void GameListDestroy P((void));
 
 /* these are in wedittags.c */
-void EditTagsPopUp P((char *tags));
+void EditTagsPopUp P((char *tags, char **dest));
 void TagsPopUp P((char *tags, char *msg));
 void TagsPopDown P((void));
 
@@ -176,6 +191,7 @@ void ClearHighlights P((void));
 void SetPremoveHighlights P((int fromX, int fromY, int toX, int toY));
 void ClearPremoveHighlights P((void));
 
+void AnimateAtomicCapture P((Board board, int fromX, int fromY, int toX, int toY));
 void ShutDownFrontEnd P((void));
 void BoardToTop P((void));
 void AnimateMove P((Board board, int fromX, int fromY, int toX, int toY));
@@ -185,6 +201,8 @@ void ThawUI P((void));
 extern char *programName;
 extern int commentUp;
 
+void GreyRevert P((Boolean grey));
+
 typedef struct FrontEndProgramStats_TAG {
     int which;
     int depth;
@@ -199,4 +217,17 @@ typedef struct FrontEndProgramStats_TAG {
 
 void SetProgramStats P(( FrontEndProgramStats * stats )); /* [AS] */
 
+void EngineOutputPopUp P((void));
+void EngineOutputPopDown P((void));
+int  EngineOutputIsUp P((void));
+int  EngineOutputDialogExists P((void));
+void EvalGraphPopUp P((void));
+void EvalGraphPopDown P((void));
+Boolean EvalGraphIsUp P((void));
+int  EvalGraphDialogExists P((void));
+
+/* these are in xhistory.c  */
+Boolean MoveHistoryIsUp P((void));
+void HistoryPopUp P((void));
+
 #endif
