@@ -5,7 +5,7 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -66,9 +66,9 @@
 
 /* Settable */
 #define FIRST_CHESS_PROGRAM	""
-#define FIRST_DIRECTORY		""
+#define FIRST_DIRECTORY		"."
 #define SECOND_CHESS_PROGRAM	""
-#define SECOND_DIRECTORY	""
+#define SECOND_DIRECTORY	"."
 
 #define CLOCK_FONT_TINY        "Arial:9.0 b"
 #define CLOCK_FONT_TEENY       "Arial:9.0 b"
@@ -211,6 +211,7 @@
 }
 
 #define SETTINGS_FILE         "winboard.ini"
+#define DEBUG_FILE            "winboard.debug"
 
 #define ICS_LOGON             "ics.ini"
 
@@ -237,7 +238,9 @@ chess.mds.mdh.se /icsport=5000\n\
 Playe&rs,players,0,1\n\
 &Games,games,0,1\n\
 &Sought,sought,0,1\n\
-|&Tell (name),tell,1,0\n\
+| ,none,0,0\n\
+Open Chat &Box (name),chat,1,0\n\
+&Tell (name),tell,1,0\n\
 M&essage (name),message,1,0\n\
 -\n\
 &Finger (name),finger,1,1\n\
