@@ -5,7 +5,7 @@
  * Massachusetts. 
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -121,8 +121,6 @@ VOID ChangeBoardSize(BoardSize newSize);
 BOOL APIENTRY MyCreateFont(HWND hwnd, MyFont *font);
 VOID ErrorPopDown(VOID);
 VOID EnsureOnScreen(int *x, int *y, int minX, int minY);
-typedef char GetFunc(void *getClosure);
-VOID ParseArgs(GetFunc get, void *cl);
 HBITMAP 
 DoLoadBitmap(HINSTANCE hinst, char *piece, int squareSize, char *suffix);
 COLORREF ParseColorName(char *name);
@@ -134,6 +132,8 @@ void InitComboStrings(HANDLE hwndCombo, char **cd);
 BOOLEAN MyLoadSound(MySound *ms);
 BOOLEAN MyPlaySound(MySound *ms);
 VOID ExitArgError(char *msg, char *badArg);
+void SaveSettings(char* name);
+BOOL BrowseForFolder( const char * title, char * path );
 
 /* Constants */
 
@@ -148,10 +148,8 @@ VOID ExitArgError(char *msg, char *badArg);
 
 /* Positions of some menu items.  Origin is zero and separator lines count. */
 /* It's gross that these are needed. */
-#define ACTION_POS 2	 /* Posn of "Action" on menu bar */
-#define OPTIONS_POS 4	 /* Posn of "Options" on menu bar */
-#define ICS_POS 4 	 /* Posn of "ICS " on Options menu */
-#define SOUNDS_POS 6     /* Posn of "Sounds" on Options menu */
+#define ACTION_POS 4	 /* Posn of "Action" on menu bar */
+#define OPTIONS_POS 6	 /* Posn of "Options" on menu bar */
 /* end grossness */
 
 extern MyFont *font[NUM_SIZES][NUM_FONTS];
@@ -173,19 +171,43 @@ extern MyFont *font[NUM_SIZES][NUM_FONTS];
 #define COPY_TMP "wbcopy.tmp"
 #define PASTE_TMP "wbpaste.tmp"
 
-/* [AS] Layout management */
-typedef struct {
-    Boolean visible;
-    int x;
-    int y;
-    int width;
-    int height;
-} WindowPlacement;
+/* variables */
+extern HINSTANCE hInst;
+extern HWND hwndMain;
+extern BoardSize boardSize;
 
+// [HGM] Some stuff to allo a platform-independent reference to windows
+// This should be moved to frontend.h in due time
+
+typedef enum {
+  W_Main, W_Console, W_Comment, W_Tags, W_GameList, 
+  W_MoveHist, W_EngineOut, NUM_WINDOWS
+} WindowID;
+
+extern WindowPlacement placementTab[NUM_WINDOWS];
+extern HWND hwndTab[NUM_WINDOWS]; // this remains pure front-end.
+
+void Translate( HWND hDlg, int id);
 VOID InitWindowPlacement( WindowPlacement * wp );
-
 VOID RestoreWindowPlacement( HWND hWnd, WindowPlacement * wp );
-
 VOID ReattachAfterMove( LPRECT lprcOldPos, int new_x, int new_y, HWND hWndChild, WindowPlacement * pwpChild );
-
 VOID ReattachAfterSize( LPRECT lprcOldPos, int new_w, int new_h, HWND hWndChild, WindowPlacement * pwpChild );
+BOOL GetActualPlacement( HWND hWnd, WindowPlacement * wp );
+
+VOID MoveHistoryPopUp();
+VOID MoveHistoryPopDown();
+VOID MoveHistorySet( char movelist[][2*MOVE_LEN], int first, int last, int current, ChessProgramStats_Move * pvInfo );
+extern HWND moveHistoryDialog;
+
+VOID EvalGraphSet( int first, int last, int current, ChessProgramStats_Move * pvInfo );
+VOID EvalGraphPopUp();
+VOID EvalGraphPopDown();
+extern HWND evalGraphDialog;
+
+extern HWND engineOutputDialog;
+
+VOID ShowGameListProc(void);
+extern HWND gameListDialog;
+
+VOID EditTagsProc(void);
+extern HWND editTagsDialog;
