@@ -1,7 +1,7 @@
 /*
  * xgamelist.h -- Game list window, part of X front end for XBoard
  *
- * Copyright 1995, 2009, 2010 Free Software Foundation, Inc.
+ * Copyright 1995, 2009, 2010, 2011 Free Software Foundation, Inc.
  * ------------------------------------------------------------------------
  *
  * GNU XBoard is free software: you can redistribute it and/or modify
@@ -27,6 +27,8 @@ void HistoryShowProc P((Widget w, XEvent *event,
 			String *prms, Cardinal *nprms));
 void HistoryPopDown P((Widget w, XtPointer client_data,
 		       XtPointer call_data));
+Boolean MoveHistoryIsUp P((void));
 
+extern Widget historyShell;
 #endif /* _XHISTL_H */
  
